/*==============================================*/
/** @file d2wrapper.hpp
 *  @brief Main Diablo II extra DLL handler.
 *
 *  This is the header of the D2 Wrapper functions.
 */
/*==============================================*/

#ifndef __D2WRAPPER_HPP__INCLUDED__
#define __D2WRAPPER_HPP__INCLUDED__

#include <windows.h>
#include "error.hpp"

#define STDCALL		__stdcall
#define FASTCALL	__fastcall

#define PATCH( F, A ) *((int*)A) = (int)&F;

/*==============================================*/
/**
 *  @brief Load a Diablo II DLL file into memory space.
 *
 *  Load a DLL called L into memory space.
 *  If the library can't be loaded, an error
 *  is thrown from file F.
 *
 */
/*==============================================*/

#define LOAD_LIBRARY( L, F )										\
if( !LoadLibrary(L) )												\
{																	\
	log_error( F, " Failed to load library " L );				    \
}																	\
else																\
{																	\
	log_info( L " loaded succesully." );						    \
}																	\

/*==============================================*/
/**
 *  @brief Unprotect Diablo II DLL.
 *
 *  Change the protection scheme of a loaded
 *  DLL called L in memory space at address A+O
 *  to allow us to customize it.
 *  If the library can't be loaded, an error
 *  is thrown from file F.
 *
 */
/*==============================================*/

#define HOOK_LIBRARY( L, A, O, F )									\
if( !VirtualProtect((LPVOID)A, O, PAGE_EXECUTE_READWRITE, &dw) )	\
{																	\
	log_error( F, " Failed to hook " L );		                    \
}																	\
else																\
{																	\
	log_info( L " succesully hooked." );							\
}																	\

/*==============================================*/
/** 
 *  @brief DLL wrapper initialisation.
 *
 *  Initialise the D2External DLL by
 *  settign up all the needed part.
 *
 *  @param pLogfilePath : Filename for the debug 
 *  log file.
 *
 */
/*==============================================*/

void STDCALL init( const char* pLogfilePath );

/*==============================================*/
/** 
 *  @brief DLL wrapper clean-up.
 *
 *  Close and release all ressoruces taken by
 *  the D2External DLL.
 *
 */
/*==============================================*/

void STDCALL release();

/*==============================================*/
/** 
 *  @brief DLL wrapper hooking.
 *
 *  Hook the D2External.dll to the global memory 
 *  space of Diablo II by retrieving the HANDLE
 *  of Diablo II application.
 *
 *  @param pModule : handle to Diablo II.
 *
 */
/*==============================================*/

void STDCALL set_module( HANDLE pModule );

/*==============================================*/
/** 
 *  @brief DLL wrapper patching.
 *
 *  Patch the various Diablo II DLL by changing
 *  the address of calling routines.
 *
 */
/*==============================================*/

void STDCALL init_patch();

static HANDLE module_handle = INVALID_HANDLE_VALUE;
static char   application_path[MAX_PATH] = "";


#endif